package ru.dbotthepony.kstarbound.client.gl.vertex

typealias QuadVertexTransformer = (AbstractVertexBuilder<*>, Int) -> AbstractVertexBuilder<*>

val EMPTY_VERTEX_TRANSFORM: QuadVertexTransformer = { it, _ -> it }

fun QuadVertexTransformer.before(other: QuadVertexTransformer): QuadVertexTransformer {
	return { a, b ->
		other.invoke(a, b)
		this.invoke(a, b)
	}
}

fun QuadVertexTransformer.after(other: QuadVertexTransformer): QuadVertexTransformer {
	return { a, b ->
		this.invoke(a, b)
		other.invoke(a, b)
	}
}

object QuadTransformers {
	fun uv(u0: Float,
		   v0: Float,
		   u1: Float,
		   v1: Float,
		   lambda: QuadVertexTransformer = EMPTY_VERTEX_TRANSFORM
	): QuadVertexTransformer {
		return transformer@{ it, index ->
			when (index) {
				0 -> it.pushVec2f(u0, v0)
				1 -> it.pushVec2f(u1, v0)
				2 -> it.pushVec2f(u0, v1)
				3 -> it.pushVec2f(u1, v1)
			}

			return@transformer lambda(it, index)
		}
	}

	fun uv(): QuadVertexTransformer {
		return transformer@{ it, index ->
			when (index) {
				0 -> it.pushVec2f(0f, 0f)
				1 -> it.pushVec2f(1f, 0f)
				2 -> it.pushVec2f(0f, 1f)
				3 -> it.pushVec2f(1f, 1f)
			}

			return@transformer it
		}
	}

	fun uv(lambda: QuadVertexTransformer): QuadVertexTransformer {
		return transformer@{ it, index ->
			when (index) {
				0 -> it.pushVec2f(0f, 0f)
				1 -> it.pushVec2f(1f, 0f)
				2 -> it.pushVec2f(0f, 1f)
				3 -> it.pushVec2f(1f, 1f)
			}

			return@transformer lambda(it, index)
		}
	}
}
